@@ -40,9 +40,14 @@
 #define C_CHAN_JOBRINGSIZE	512
 
 
-/* Binary message header magic number (32 bit) */
+/* Binary message header magic number (32 bit); REQMAGIC is used for requests,
+ * normally from core to module, but can also be the other way around, and in
+ * that case the core will decode the request using the dictionary space 
+ * C_DS_MODGROUND instead of meta_binmsgtoav; REPMAGIC is used for responses,
+ * normally from modules to core, but can also be the other way around. */
 
-#define C_BINMSG_MAGIC		0xdeadbeef
+#define C_BINMSG_REQMAGIC	0xbeefdead
+#define C_BINMSG_REPMAGIC	0xdeadbeef
 
 
 /* Maximum shared secret length */
@@ -65,7 +70,8 @@
    These must match the values in the dictionary! You can give them any 
    names you want in the dictionary, as long as you use the values below. */
 
-#define C_DS_RAD_PKT		1	/* Space that holds whole packet */
+#define C_DS_GROUND		0	/* Ground space, holding whole packet */
+#define C_DS_RAD_PKT		1	/* Space that holds items below */
 
 #define C_DI_CODE		0	/* Code (in above space) */
 #define C_DI_IDENTIFIER		1	/* Identifier (in above spc) */
@@ -76,14 +82,24 @@
 #define C_DV_CODE_REJECT	3
 #define C_DV_CODE_ACCTRESP	5
 
+#define C_DS_RAD_ATR		2	/* Space that holds attributes */
+
+#define C_DI_MSG_AUTH		80	/* Message-Authenticator attribute.
+					   We sign packet here after encoding
+					   if it's present on the reply list. */
+
 #define C_DS_INTERNAL		100	/* Space that holds internal attrs */
 
 #define C_DI_TIMESTAMP		10	/* Timestamp of request */
 #define C_DI_SECRET		11	/* Shared secret to sign response */
+#define C_DI_LOG_LINE		12	/* Line logged at end of expression */
+
 #define C_DI_IP_SOURCE		16	/* Source IP address of request */
 #define C_DI_IP_DEST		17	/* Destination address of request */
 #define C_DI_UDP_SOURCE		18	/* Source UDP port of request */
 #define C_DI_UDP_DEST		19	/* Destination port of request */
+#define C_DI_SOURCE		20	/* Source of request (name) */
+#define C_DI_DEST		21	/* Destination of request (name) */
 
 #define C_DI_ADDR		25	/* Config: IP address to bind on */
 #define C_DI_PORT		26	/* Config: UDP port to bind on */
@@ -164,3 +180,8 @@
 #define C_DI_USER_NAME		1	/* Used by eg. RadLDAP */
 #define C_DI_USER_PASSWORD	2	/* Used by eg. RadLDAP */
 
+
+/* Attributes for modules that pass a transaction ID around. In DS_INTERNAL */
+
+#define C_DI_TID		100	/* Used by eg. EAPTLS */
+
