@@ -59,7 +59,7 @@ typedef struct ring {
 
 /* Counting function for peeks */
 
-typedef ssize_t (*RINGCNTFUNC)(char *s, ssize_t len, va_list ap);
+typedef ssize_t (*RINGCNTFUNC)(char *s, ssize_t len, ssize_t seen, va_list ap);
 
 
 /*
@@ -151,9 +151,9 @@ ssize_t ring_count(RING *r, RINGCNTFUNC func, ...);
 
 /* Counting functions for use with ring_count */
 
-ssize_t _ring_cnt_strspn(char *s, ssize_t len, va_list ap);
-ssize_t _ring_cnt_strcspn(char *s, ssize_t len, va_list ap);
-ssize_t _ring_cnt_strncmp(char *s, ssize_t len, va_list ap);
+ssize_t _ring_cnt_strspn(char *s, ssize_t len, ssize_t seen, va_list ap);
+ssize_t _ring_cnt_strcspn(char *s, ssize_t len, ssize_t seen, va_list ap);
+ssize_t _ring_cnt_strncmp(char *s, ssize_t len, ssize_t seen, va_list ap);
 
 
 /*
