@@ -86,17 +86,20 @@ typedef struct stdop {
 /* The standard operators we know */
 
 
-#define STDOP_CNT	73
+#define STDOP_CNT	85
 
 static STDOP stdops[STDOP_CNT] = {
+	/* Statements */
 	{ "halt",	 NON, 32, L,   ANY, NON,   INT, OP_HALT },
 	{ "abort",	 NON, 32, L,   ANY, NON,   INT, OP_ABORT },
 	{ "accept",	 NON, 32, L,   ANY, NON,   INT, OP_ACCEPT },
 	{ "reject",	 NON, 32, L,   ANY, NON,   INT, OP_REJECT },
 	{ "acctresp",	 NON, 32, L,   ANY, NON,   INT, OP_ACCTRESP },
-	{ "delall",	 NON, 32, L,   ANY, ANY,   INT, OP_DELALLAV },
-	{ "del",	 NON, 32, L,   ANY, ANY,   INT, OP_DELAV },
-	{ "moveall",	 NON, 32, L,   ANY, ANY,   INT, OP_MOVEALLAV },
+	/* High precedence unary prefix operators. Unary prefix is always R. */
+	{ "delall",	 NON, 32, R,   ANY, ANY,   INT, OP_DELALLAV },
+	{ "del",	 NON, 32, R,   ANY, ANY,   INT, OP_DELAV },
+	{ "moveall",	 NON, 32, R,   ANY, ANY,   INT, OP_MOVEALLAV },
+	{ "all",	 NON, 32, R,   ANY, ANY,   STR, OP_JOINSTR },
 	/* Typecasts / auto-conversion (uses autodetected format for ints) */
 	{ "toint",	 ANY, 30, L,   INT, NON,   INT, OP_NOP },
 	{ "toip",	 ANY, 30, L,   IPA, NON,   IPA, OP_NOP },
@@ -112,14 +115,21 @@ static STDOP stdops[STDOP_CNT] = {
 	{ "todec",	 ANY, 30, L,   INT, NON,   STR, OP_ORD2DECSTR },
 	{ "tohex",	 ANY, 30, L,   INT, NON,   STR, OP_ORD2HEXSTR },
 	{ "toraw",	 ANY, 30, L,   INT, NON,   STR, OP_ORD2RAWSTR },
+	/* Other unary postfix operators */
 	{ "exists",	 ANY, 30, L,   ANY, NON,   INT, OP_EXISTS },
+	/* Other unary prefix operators. If in doubt, use prefix, not postfix */
 	{ "~",		 NON, 28, R,   ANY, INT,   INT, OP_NOT },
 	{ "-",		 NON, 28, R,   ANY, INT,   INT, OP_NEG },
+	{ "dictencode",	 NON, 28, R,   ANY, ANY,   STR, OP_DICTENC },
 	{ "!",		 NON, 28, R,   ANY, ANY,   INT, OP_BOOLNOT },
 	{ "not",	 NON, 28, R,   ANY, ANY,   INT, OP_BOOLNOT },
 	{ "no",		 NON, 28, R,   ANY, ANY,   INT, OP_BOOLNOT },
+	{ "random",	 NON, 28, R,   ANY, ANY,   STR, OP_RANDOM },
 	{ "md5",	 NON, 28, R,   ANY, STR,   STR, OP_MD5 },
 	{ "hex",	 NON, 28, R,   ANY, STR,   STR, OP_HEX },
+	{ "upper",	 NON, 28, R,   ANY, STR,   STR, OP_UPPER },
+	{ "lower",	 NON, 28, R,   ANY, STR,   STR, OP_LOWER },
+	/* Binary operators */
 	{ "*",		 ANY, 25, L,   INT, INT,   INT, OP_MUL },
 	{ "/",		 IPA, 25, L,   ANY, INT,   IPA, OP_CIDRMASK },
 	{ "/",		 ANY, 25, L,   INT, INT,   INT, OP_DIV },
@@ -136,6 +146,12 @@ static STDOP stdops[STDOP_CNT] = {
 	{ "^",		 STR, 22, L,   ANY, STR,   STR, OP_XORSTR },
 	{ "&",		 ANY, 21, L,   INT, INT,   INT, OP_AND },
 	{ "|",		 ANY, 20, L,   INT, INT,   INT, OP_OR },
+	{ "asmac", 	 ANY, 18, L,   STR, STR,   STR, OP_STR2MAC },
+	{ "papdecrypt",  ANY, 18, L,   STR, STR,   STR, OP_PAPDECR },
+	{ "papencrypt",  ANY, 18, L,   STR, STR,   STR, OP_PAPENCR },
+	{ "unixcrypt",	 ANY, 18, L,   STR, STR,   STR, OP_UNIXCRYPT },
+	{ "hmacmd5",	 ANY, 18, L,   STR, STR,   STR, OP_HMACMD5 },
+	{ "dictdecode",  ANY, 18, L,   STR, ANY,   INT, OP_DICTDEC },
 	/* Convert to run-time specified format (only implemented for dates) */
 	{ "as", 	 ANY, 18, L,   DAT, STR,   STR, OP_ORD2DFMSTR },
 	{ "beforefirst", ANY, 16, L,   STR, STR,   STR, OP_BF },
@@ -165,6 +181,7 @@ static STDOP stdops[STDOP_CNT] = {
 	{ ":=",		 IPA,  7, R,   ANY, IPA,   IPA, OP_REPLACEAV },
 	{ ":=",		 DAT,  7, R,   ANY, DAT,   DAT, OP_REPLACEAV },
 	{ ":=",		 STR,  7, R,   ANY, STR,   STR, OP_REPLACEAV },
+	{ "pokedwith",	 STR,  7, R,   ANY, STR,   STR, OP_POKEAV },
 	{ ",",		 ANY,  1, L,   ANY, NON,   NON, OP_POP }
 	};
 
@@ -394,7 +411,7 @@ static ssize_t reccompile(META *m, IFACE *ifaces, TEXT *source,
 		if (c == '(' && context == NON) {
 			S(1);
 			l = reccompile(m, ifaces, source, &o, bufsize - ret, 0,
-				       req_ctx, ctx_item, &context);
+				       ANY, ctx_item, &context);
 			if (l == -1) return -1;
 			ret += l;
 			continue;
