@@ -33,7 +33,7 @@ char langdisasm_id[] = "LANGDISASM - Copyright (C) 2001 Emile van Bergen.";
  */
 
 
-#define OPCCNT 69
+#define OPCCNT 81
 
 struct {
 	int op;
@@ -79,8 +79,12 @@ struct {
 	{ OP_LO, "LO" },
 	{ OP_MD5, "MD5" },
 	{ OP_HEX, "HEX" },
+	{ OP_UPPER, "UPPER" },
+	{ OP_LOWER, "LOWER" },
+	{ OP_RANDOM, "RANDOM" },
 	{ OP_XORSTR, "XORSTR" },
 	{ OP_CONCAT, "CONCAT" },
+	{ OP_DICTENC, "DICTENC" },
 
 	{ OP_GESTR, "GESTR" },
 	{ OP_LESTR, "LESTR" },
@@ -96,6 +100,12 @@ struct {
 	{ OP_ORD2IPASTR, "ORD2IPASTR" },
 	{ OP_ORD2DATSTR, "ORD2DATSTR" },
 	{ OP_ORD2DFMSTR, "ORD2DFMSTR" },
+	{ OP_STR2MAC, "STR2MAC" },
+	{ OP_PAPDECR, "PAPDECR" },
+	{ OP_PAPENCR, "PAPENCR" },
+	{ OP_UNIXCRYPT, "UNIXCRYPT" },
+	{ OP_HMACMD5, "HMACMD5" },
+	{ OP_DICTDEC, "DICTDEC" },
 
 	{ OP_OCTSTR2ORD, "OCTSTR2ORD" },
 	{ OP_DECSTR2ORD, "DECSTR2ORD" },
@@ -112,9 +122,11 @@ struct {
 
 	{ OP_ADDAV, "ADDAV" },
 	{ OP_REPLACEAV, "REPLACEAV" },
+	{ OP_POKEAV, "POKEAV" },
 	{ OP_DELAV, "DELAV" },
 	{ OP_DELALLAV, "DELALLAV" },
 	{ OP_MOVEALLAV, "MOVEALLAV" },
+	{ OP_JOINSTR, "JOINSTR" },
 
 	{ OP_CALLIFACE, "CALLIFACE" },
 
@@ -160,7 +172,7 @@ void lang_disassemble(META *m, INSN *code, ssize_t codelen)
 			*o++ = '\"';
 			o += meta_atoprt((char *)(i + 1), i->imm.d.str_len,
 					 0, 0, 0, 0, 
-					 o, 128);
+					 o, sizeof(buf) - 4 - (o - buf));
 			*o++ = '\"';
 			i += i->imm.d.disp;
 			break;
