@@ -142,7 +142,7 @@ void parseoptions(int argc, char **argv)
 		if (n == -1 || dup2(n, 2) == -1 || fcntl(2, F_SETFD, 0) == -1 ||
 		    close(n) == -1) {
 			msg(F_MISC, L_ERR, "parseoptions: ERROR: Could not open"
-					   "'%s' to redirect stderr to: %s!\n",
+					   " '%s' to redirect stderr to: %s!\n",
 			    logfile, strerror(errno));
 			usage(0);
 		}
@@ -228,11 +228,12 @@ int main(int argc, char **argv)
 	sa.sa_flags = SA_NOCLDSTOP + SA_RESTART; sa.sa_handler = sighandler; 
 	sigaction(SIGHUP, &sa, 0); sigaction(SIGINT, &sa, 0);
 	sigaction(SIGQUIT, &sa, 0); sigaction(SIGTERM, &sa, 0);
-	sigaction(SIGCHLD, &sa, 0);
+	sigaction(SIGCHLD, &sa, 0); 
+	sa.sa_handler = SIG_IGN; sigaction(SIGPIPE, &sa, 0); 
 #else
 	signal(SIGHUP, sighandler); signal(SIGINT, sighandler);
 	signal(SIGQUIT, sighandler); signal(SIGTERM, sighandler); 
-	signal(SIGCHLD, sighandler);
+	signal(SIGCHLD, sighandler); signal(SIGPIPE, SIG_IGN);
 #endif
 
 	/* Go to background */
@@ -379,7 +380,7 @@ int main(int argc, char **argv)
 		for(s = c->sources; s; s = s->next) if (FD_ISSET(s->fd, &rfds)){
 
 			/* Create new job from this source */
-			j = job_new(s, t); if (!j) continue;
+			j = job_new_fromsock(s, t); if (!j) continue;
 
 			/* Run the expression on this job till we have replied
 			   or we got an interface call */
