@@ -49,7 +49,7 @@
 #include <metaops.h>	/* For META_AV etc. */
 
 #include <ringbuf.h>	/* Ring buffer for stdin */
-#include <misc.h>	/* For encrypt_attr_pap() */
+#include <misc.h>	/* For encrypt_attr_pap(), get_random_data */
 #include <md5.h>
 #include <debug.h>
 
@@ -180,7 +180,7 @@ struct radcfg {
 
 	/* Cached information from dictionary */
 
-	META_SPC *ds_rad_pkt;
+	META_SPC *ds_ground, *ds_rad_pkt;
 	META_ITEM *di_int, *di_rad_code, *di_rad_id, *di_rad_len, *di_rad_auth;
 	META_ITEM *di_user_name, *di_user_pass, *di_chap_pass, *di_chap_chal; 
 	META_ITEM *di_nas_ip, *di_acct_dly, *di_query_id, *di_tgt_srv;
@@ -235,26 +235,6 @@ void *safe_malloc(size_t size)
 }
 
 
-/* Get reasonably good random data */
-
-#define RANDOM_DEV	"/dev/urandom"
-
-static void get_random_data(char *p, ssize_t len)
-{
-#ifdef RANDOM_DEV
-	int fd;
-	ssize_t n;
-
-	if ((fd = open(RANDOM_DEV, O_RDONLY)) != -1 &&
-	    (n = read(fd, p, len)) == len &&
-	    close(fd) != -1) return;
-
-	msg(F_MISC, L_NOTICE, "Warning: No data from " RANDOM_DEV ", using rand()\n");
-#endif
-	while(len-- > 0) *p++ = (char)(256.0 * rand() / (RAND_MAX + 1.0));
-}
-
-
 /***************************************
  * INPUT FROM SOCKET, OUTPUT TO STDOUT *
  ***************************************/
@@ -314,10 +294,10 @@ void query_reply(struct radcfg *c, struct radquery *qry, int status)
 		memset(&av, 0, sizeof(av));
 		av.i = c->di_int;
 		av.l = status;
-		n += meta_avtomsg(c->m, &av, buf + n, OUT_BUFLEN - n, c->ioflags | AVMSG_ONESHOT, 0);
+		n += meta_avtomsg(c->m, &av, buf + n, OUT_BUFLEN - n, c->ioflags | AVMSG_ONESHOT, 0, 0);
 	}
 	if (qry) {
-		if (qry->rephead) n += meta_avtomsg(c->m, qry->rephead, buf + n, OUT_BUFLEN - n, c->ioflags, 0);
+		if (qry->rephead) n += meta_avtomsg(c->m, qry->rephead, buf + n, OUT_BUFLEN - n, c->ioflags, 0, 0);
 		query_del(c, qry);
 	}
 	if (n >= OUT_BUFLEN) {
@@ -362,7 +342,7 @@ void sock_handle_read(struct radcfg *c, time_t t)
 
 	/* Decode */
 
-	head = meta_decode(c->m, c->ds_rad_pkt, 0, pkt, n, &tail);
+	head = meta_decode(c->m, c->ds_ground, 0, pkt, n, &tail);
 	if (!head) {
 	    msg(F_RECV, L_ERR, "Dropping answer: Could not decode response!\n");
 	    return;
@@ -392,14 +372,8 @@ void sock_handle_read(struct radcfg *c, time_t t)
 	    msg(F_RECV, L_NOTICE, "Note: Length in RADIUS packet (%d) differs from that of UDP packet (%d).\n", rad_len, n);
 	}
 
-	/* Check the RADIUS ID and find the request this answer is for */
+	/* Find the request this answer is for in the ring/hash */
 
-	if ((rad_id & ~MAX_OUTREQMASK) != 
-	    ((getpid() << MAX_OUTREQBITS) & c->max_idmask)) {
-	    msg(F_RECV, L_ERR, "Dropping answer: ID %d was never used by me!\n",
-		rad_id);
-	    return;
-	}
 	n = rad_id & MAX_OUTREQMASK;
 	r = &c->req[n];
 	if (r->pktlen <= 0) {
@@ -407,6 +381,11 @@ void sock_handle_read(struct radcfg *c, time_t t)
 		n, rad_id);
 	    return;
 	}
+	if (rad_id != r->id) {
+	    msg(F_RECV, L_ERR, "Dropping answer: ID %d was never used by me!\n",
+		rad_id);
+	    return;
+	}
 
 	/* Check the authenticator by substituting the original request 
 	   authenticator for the response authenticator, appending the
@@ -477,8 +456,9 @@ int new_req(struct radcfg *c, time_t t, struct radquery *qry,
 {
 	struct radreq *r;
 	META_AV *av, *head, *tail, *tree;
+	META_ORD n;
 	md5_state_t md5ctx;
-	int ret, n;
+	int ret;
 
 	/* Take given slot or next slot in ring and check if free */
 
@@ -514,7 +494,7 @@ int new_req(struct radcfg *c, time_t t, struct radquery *qry,
 		meta_addav(&head, &tail, 0, 0, av);
 	}						/* Add RADIUS ID */
 	r->id = av->l;
-	if (qry->missing & A_RAD_AUTH) {
+	if (qry->missing & A_RAD_AUTH) {		/* (never when acct). */
 		av = safe_malloc(sizeof(META_AV)); 
 		memset(av, 0, sizeof(META_AV));
 		av->i = c->di_rad_auth; 
@@ -522,6 +502,15 @@ int new_req(struct radcfg *c, time_t t, struct radquery *qry,
 		get_random_data(av->p, av->l);
 		meta_addav(&head, &tail, 0, 0, av);
 	}						/* Add authenticator */
+	else if (qry->given_auth) {
+		/* If we didn't need to generate a request authenticator before
+		   encoding the packet (because we already have one or we're
+		   doing accounting) *and* that is because we were given one in
+		   the query, copy it to the request authenticator member to 
+		   make sure the response check and encrypt_attr_pap (if done)
+		   use the given one. */
+		memcpy(r->auth, qry->given_auth->p, MIN(sizeof(r->auth), qry->given_auth->l)); 
+	}
 
 	/* Add query list */
 
@@ -561,12 +550,12 @@ int new_req(struct radcfg *c, time_t t, struct radquery *qry,
 
 	/* Build encapsulation tree from list and a packet from that */
 
-	meta_buildtree(head, &tree, c->ds_rad_pkt);
-	r->pktlen = meta_encode(c->ds_rad_pkt, r->pkt, C_MAX_PKTSIZE, tree, 0);
+	meta_buildtree(head, &tree, c->ds_ground);
+	r->pktlen = meta_encode(c->ds_ground, r->pkt, C_MAX_PKTSIZE, tree, 0);
 
 	/* Free temporary tree and list parts */
 
-	meta_freeavlist(tree); tree = 0;	/* Free tree */
+	meta_freeavtree(tree); tree = 0;	/* Free tree */
 	if (qry->reqhead->prev) {
 		qry->reqhead->prev->next = 0;
 		meta_freeavlist(head); head = 0;	
@@ -584,24 +573,17 @@ int new_req(struct radcfg *c, time_t t, struct radquery *qry,
 		return -1;
 	}
 
-	/* If acct, put packet signature or given auth in r->auth and packet */
+	/* If we're doing accounting and we didn't already have a signature, 
+	   sign the packet now and remember the signature in r->auth so that we
+	   can check the response. */
 
-	if (qry->acct) {
-	    if (qry->given_auth) {
-		if (qry->given_auth->l >= 16) {
-		    memcpy(r->auth, qry->given_auth->p, sizeof(r->auth));
-		} else {
-		    memset(r->auth, 0, sizeof(r->auth));
-		    memcpy(r->auth, qry->given_auth->p, qry->given_auth->l);
-		}
-	    } else {
+	if (qry->acct && !qry->given_auth) {
 		md5_init(&md5ctx);
 		md5_append(&md5ctx, r->pkt, r->pktlen);
 		md5_append(&md5ctx, srv->secret, srv->secretlen);
 		md5_finish(&md5ctx, r->auth);
-	    }
-	    memcpy(r->pkt + c->di_rad_auth->val_ofs, r->auth, 
-		   c->di_rad_auth->val_size);
+		memcpy(r->pkt + c->di_rad_auth->val_ofs, r->auth, 
+		       c->di_rad_auth->val_size);
 	}
 
 	return ret;
@@ -1081,6 +1063,7 @@ void parseoptions(int argc, char **argv, struct radcfg *c)
 	}
 
 	msg_init(-1, c->quiet);
+	msg_settag("C", 1);
 }
 
 
@@ -1094,32 +1077,46 @@ void opendict(struct radcfg *c)
 
 	/* We cache some dictionary items for speed and convenience */
 
-	if (!(c->ds_rad_pkt   = meta_getspcbyname(c->m, "RAD-PKT")) ||
-	    !(c->di_int       = meta_getitembyname(c->m, 0, "int", 0)) ||
+	if (!(c->ds_ground    = meta_getspcbynr(c->m, 0)) ||
+	    !(c->ds_rad_pkt   = meta_getspcbyname(c->m, "RAD-PKT")) ||
+	    !(c->di_int       = meta_getitembyname(c->m, 0, "int", 
+						   META_VND_ANY)) ||
 	    !(c->di_rad_code  = meta_getitembyname(c->m, c->ds_rad_pkt,
-						   "RAD-Code", 0)) ||
+						   "RAD-Code", 
+						   META_VND_ANY)) ||
 	    !(c->di_rad_id    = meta_getitembyname(c->m, c->ds_rad_pkt,
-						   "RAD-Identifier", 0)) ||
+						   "RAD-Identifier", 
+						   META_VND_ANY)) ||
 	    !(c->di_rad_len   = meta_getitembyname(c->m, c->ds_rad_pkt,
-						   "RAD-Length", 0)) ||
+						   "RAD-Length", 
+						   META_VND_ANY)) ||
 	    !(c->di_rad_auth  = meta_getitembyname(c->m, c->ds_rad_pkt,
-						   "RAD-Authenticator", 0)) ||
+						   "RAD-Authenticator", 
+						   META_VND_ANY)) ||
 	    !(c->di_user_name = meta_getitembyname(c->m, 0, 
-			    			   "User-Name", 0)) ||
+			    			   "User-Name", 
+						   META_VND_ANY)) ||
 	    !(c->di_user_pass = meta_getitembyname(c->m, 0, 
-			    			   "User-Password", 0)) ||
+			    			   "User-Password", 
+						   META_VND_ANY)) ||
 	    !(c->di_chap_pass = meta_getitembyname(c->m, 0, 
-			    			   "CHAP-Password", 0)) ||
+			    			   "CHAP-Password", 
+						   META_VND_ANY)) ||
 	    !(c->di_chap_chal = meta_getitembyname(c->m, 0, 
-			    			   "CHAP-Challenge", 0)) ||
+			    			   "CHAP-Challenge", 
+						   META_VND_ANY)) ||
 	    !(c->di_nas_ip    = meta_getitembyname(c->m, 0, 
-			    			   "NAS-IP-Address", 0)) ||
+			    			   "NAS-IP-Address", 
+						   META_VND_ANY)) ||
 	    !(c->di_acct_dly  = meta_getitembyname(c->m, 0, 
-			    			   "Acct-Delay-Time", 0)) ||
+			    			   "Acct-Delay-Time", 
+						   META_VND_ANY)) ||
 	    !(c->di_query_id  = meta_getitembyname(c->m, 0, 
-			    			   "Radclient-Query-Id", 0)) ||
+			    			   "Radclient-Query-Id", 
+						   META_VND_ANY)) ||
 	    !(c->di_tgt_srv   = meta_getitembyname(c->m, 0, 
-			    			   "Target-Server", 0)) ||
+			    			   "Target-Server", 
+						   META_VND_ANY)) ||
 	    !(c->dv_auth_req  = meta_getvalbyname(c->m, c->di_rad_code,
 			    			  "Access-Request")) ||
 	    !(c->dv_acct_req  = meta_getvalbyname(c->m, c->di_rad_code,
@@ -1206,7 +1203,7 @@ int main(int argc, char **argv)
 		_exit(EXIT_INTERN);
 	}
 
-	if (!c->quiet) { 
+	if (!c->quiet && !getenv("RADIUSINTERFACEVERSION")) { 
 		fprintf(stderr, "Enter A/V pairs, one per line, terminated by an empty line or ^D.\n"); 
 		fflush(stderr); 
 	}
